#!/bin/bash
# (c) Copyright 2007-2014 Hewlett Packard Development Company, L.P.
# Version A.12.00.00

###############################################################################
#
# Sendmail Toolkit Main Script:
#
# This script contains SENDMAIL's defined environment variables and functions 
# that are used in the package control script.
#
# To be a part of a package, this script (and along with its Monitoring
# Script and the User Configuration Script) has to be placed in the package 
# directory.
#
# The package control script will check for this script in its package
# directory.  If it exists, the package control script will continue to
# proceed and will include it.
#  
# Implementation: 
#
# 1) Modifications were made to an existing toolkit template's code to 
#    specifically handle the Sendmail package. Error conditions are added.
#    Depends on the errors, this script exits with different error values.
#
# 2) A new function, check_return, is added to handle the return value from
#    various functions.  This function is similar to the test_return function
#    in the package control script.  All the existing calls to test_return are
#    changed to check_return.
# 
# 3) Code were added to the main body of this script to start the functions. 
#  
# Exit value definition:
#
# 0 - status ok 
#
# 1 - a failure occurs in one of the functions.  Return control to the package
#     control script.  The package control script needs to do additional 
#     cleanup such as umount_fs, deactivate_volume_group,.. before exiting  
#     with an error.
#
# REVISION HISTORY:
#     Version       mm/dd/yyyy   Changes
#     A.03.00       07/10/2006   1) Changed kill -TERM to kill -15 as kill -TERM
#	 			    does not seem to be working on Redhat4
#
#     A.04.00       03/27/2008   1) Added CONF_DIR variable
#				    Removed start and halt service invocation
#				    and definitions	
#	
#     A.04.02.01    04/30/2012   1) Added variables HA_SERVER_CMD, HA_SERVER_DIR
#                                   HA_SERVER_PROC.
#                                2) Removed check of DAEMON variable for Sendmail
#                                   server start.  
#     A.12.00.00    09/15/2013   1) Removed references to hasendmail.conf and TKIT_DIR
#                                2) Added idempotent operation support
######################################################################################


PATH=/bin:/sbin:/usr/bin:/usr/sbin:/bin:

# Either "start" or "stop"
ACTION=$1       

#
# Determine the package directory
#
PKG_DIR=${0%/*}
if [[ $PKG_DIR = ${0##*/} ]]; then
    PKG_DIR=`pwd`
fi

CONF_DIR=$PKG_DIR
HA_SERVER_CMD="/usr/sbin/sendmail"
HA_SERVER_DIR="/etc/mail"
HA_SERVER_PROC="sendmail: accepting connections"

# Assigning default value 30 to MONITOR_INTERVAL.
if [ -z $MONITOR_INTERVAL ]; then
    MONITOR_INTERVAL=30
fi

#
# START OF RUN FUNCTIONS
#

#
# This function starts the SENDMAIL daemon and returns with ret_code status:
# ret_code: 0 - ok status; 1 - error status 
# The pid file in /var/run/sendmail.pid is automatically updated when 
# the SENDMAIL daemon is successfully launched.
#
function start_server
{
    echo "$(date '+%b %e %T') - Node \"$(hostname)\": Starting SENDMAIL server."

    # Start SENDMAIL daemon if it is not running 

    local RET_CODE=0

    pidlist=`pidofproc "$HA_SERVER_PROC" `

    if [[ -z $pidlist ]]; then

        echo -e "\tVerified no other Sendmail daemon running."

        # rebuild the db files in /etc/mail directory for
        # virtusertable access domaintable mailertable

        echo -e "\tStart rebuilding Sendmail db files."

        for table in ${HA_SERVER_DB_FILES}; do

           if [[ -f ${HA_SERVER_DIR}/$table ]]; then
               makemap hash ${HA_SERVER_DIR}/$table < ${HA_SERVER_DIR}/$table

           fi
        done

        # Start the Sendmail daemon and its option QUEUE to process those
        # queued mail in the mail queue directories. -q$QUEUE tells Sendmail
        # to wake up once every $QUEUE interval to process the queue.


        echo -e "\tStarting the Sendmail daemon -bd -q$QUEUE."

        ${HA_SERVER_CMD} -bd -q$QUEUE
        RET_CODE=$?
        sleep 3


    else

        echo "WARNING: Sendmail is already running on this node."
        exit 0 

    fi

    check_return $RET_CODE 1

echo -e "\tEnd of start_server"

}


# This function kills SENDMAIL daemon.
#
function stop_server
{

    echo "$(date '+%b %e %T') - Node \"$(hostname)\": Stopping SENDMAIL server"
  
    local RET_CODE=0

    # Kill the Sendmail daemon

    echo -e "\tKilling Sendmail daemon."

    killproc "$HA_SERVER_PROC" -15

    RET_CODE=$?

    # Remove Sendmail pid file (/var/run/sendmail.pid)

    if [[ -r ${HA_SERVER_PID} ]]; then

        echo -e "\tRemoving Sendmail file ${HA_SERVER_PID}."

        rm -f ${HA_SERVER_PID}
    fi

    check_return $RET_CODE 4

    echo -e "\tEnd of stop_server."

}


#
# Find all pid of a process
#
function pidofproc
{
    pidof "$1"
}  


#
# kill the named process
# Parameter:	$1: process name
#		$2: kill signal
function killproc 

{
    local RET_CODE=0
    local kill_sig="-15"
    [ -n $2 ] && kill_sig=$2 
    pidlist=`pidof "$1"`
    if [[ -n $pidlist ]]; then
        for pid in "$pidlist"; do
            kill $kill_sig $pid
            if [ $? -ne 0 ]; then
                RET_CODE=1
            fi
        done
    fi
    return $RET_CODE
}

#
# Check return value of functions and then exit.
# Parameter:    $1: exit code of a previous command
#               $2: function ID
#
function check_return
{
    if [[ $1 != 0 ]] 
    then
	case $2 in
	    1)
		echo "ERROR: Function start_server"
                echo "ERROR: Failed to start SENDMAIL daemon"
                stop_server
                exit 1
                ;;

	    4)
                echo "ERROR: Function stop_server"
                echo "ERROR: Failed to stop SENDMAIL daemon"
                exit_value=1
                ;;

           31)
                echo "ERROR: Function customer_sendmail_defined_run_cmds"
                echo "ERROR: Failed to run customer commands"
                exit_value=1
                ;;

           32)
                echo "ERROR: Function customer_sendmail_defined_halt_cmds"
                echo "ERROR: Failed to halt customer commands"
                exit_value=1
                ;;

	    *)
		echo "ERROR: Failed, unknown error."
		;;
	esac
    fi
}


#
# FUNCTION STARTUP SECTION.  
#

exit_value=0

#
# Test to see if we are being called to run the package, or halt the package.
#
if [[ "$ACTION" = "start" ]]
then

    start_server


elif [[ "$ACTION" = "stop" ]]
then

    
    stop_server
fi

exit $exit_value

 
